package com.miaxis.wx.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.miaxis.common.constant.Constants;
import com.miaxis.common.utils.StringUtils;
import com.miaxis.feign.dto.WxMessageCusom;
import com.miaxis.feign.service.IWxSendService;
import com.miaxis.wx.service.IWxGzhService;
import com.miaxis.wx.service.WxService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Formatter;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * 微信公众号业务层
 * @author wwl
 * @version 1.0
 * @date 2021/10/20 14:57
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class WxGzhServiceImpl implements IWxGzhService {

    private final RedisTemplate redisTemplate;

    private final IWxSendService wxSendService;



    private final WxService wxService;


    @Value("${zzgzh.appid}")
    private String appid;

    @Value("${zzgzh.appSecret}")
    private String secret;


    /**
     * 获取微信公众号token
     * @return
     */
    @Override
    public String getGzhToken() {
        String token = "";
        Integer expiresIn;
        if (redisTemplate.hasKey(Constants.GZH_MESSAGE_TOKEN)){
            token = (String)redisTemplate.opsForValue().get(Constants.GZH_MESSAGE_TOKEN);
        }
        if (StringUtils.isEmpty(token)){
            String result = wxSendService.getAccessToken("client_credential",appid,secret);
            JSONObject json = JSONObject.parseObject(result);
            token = json.getString("access_token");
            expiresIn = 1800;
//            expiresIn = json.getIntValue("expires_in");
//            if (expiresIn <= 0){
//                expiresIn = 7000;
//            }

            redisTemplate.opsForValue().set(Constants.GZH_MESSAGE_TOKEN,token,expiresIn, TimeUnit.SECONDS);
        }

        return token;
    }

    //判定是否失效
    private boolean notEffect(String token) {
        String res = wxSendService.userInfo( token, "ovKTX5-FKLF6_sgTtCIXpG_lz3PY", "zh_CN");
        JSONObject jsonObject = JSONObject.parseObject(res);
        return jsonObject.getString("unionid") ==null;

    }


    /**
     * 微信推送消息处理
     * @param request
     * @return
     */
    @Override
    public String handlePublicMsg(HttpServletRequest request) {

        return null;
    }




    /**
     * 发送客服消息
     * @param fromUserName
     * @param token
     */
    private void messageCustomSend(String fromUserName,String token,String content){

        WxMessageCusom wxMessageCusom = new WxMessageCusom();
        wxMessageCusom.setTouser(fromUserName);
        wxMessageCusom.setMsgtype("text");
        HashMap<String, Object> contentMap = new HashMap<>();
        contentMap.put("content",content);
        wxMessageCusom.setText(contentMap);
        String send = wxSendService.messageCustomSend(token, wxMessageCusom);
        JSONObject jsonObject = JSONObject.parseObject(send);
        log.info("回复客服消息：[{}],响应信息：[{}]",contentMap,jsonObject);
    }


    @Override
    public JSONObject getJspapi(String url) {

        //1、获取AccessToken
        String gzhToken = this.getGzhToken();
        //2、获取Ticket
        String jsapiStr = wxService.getticket(gzhToken, "jsapi");
        //3、时间戳和随机字符串
//        String nonce_str = RandomStringUtils.randomAlphanumeric(16);
//        long timestamp =  System.currentTimeMillis()/1000;
        JSONObject jsapiData = JSONObject.parseObject(jsapiStr);
        String ticket= jsapiData.getString("ticket");
        Map<String, String> ret = sign(ticket, "https://jpcj-h5.zzxcx.net/mini1");



//        //4、获取url
//        String str = "jsapi_ticket="+jsapiStr+"&noncestr="+nonce_str+"&timestamp="+timestamp+"&url="+url;
//
//        //6、将字符串进行sha1加密
//        String signature = SHA1Util.SHA1(str);
        JSONObject resultJSONObject = new JSONObject();
        resultJSONObject.put("ticket",ticket);
        resultJSONObject.put("appId",appid);
        resultJSONObject.put("timestamp",ret.get("timestamp"));
        resultJSONObject.put("nonceStr",ret.get("nonceStr"));
        resultJSONObject.put("signature",ret.get("signature"));
        return resultJSONObject;
    }

    public static Map<String, String> sign(String jsapi_ticket, String url) {
        Map<String, String> ret = new HashMap<String, String>();
        String nonce_str = create_nonce_str();
        String timestamp = create_timestamp();
        String string1;
        String signature = "";

        //注意这里参数名必须全部小写，且必须有序
        string1 = "jsapi_ticket=" + jsapi_ticket +
                "&noncestr=" + nonce_str +
                "&timestamp=" + timestamp +
                "&url=" + url;
        System.out.println(string1);

        try
        {
            MessageDigest crypt = MessageDigest.getInstance("SHA-1");
            crypt.reset();
            crypt.update(string1.getBytes("UTF-8"));
            signature = byteToHex(crypt.digest());
        }
        catch (NoSuchAlgorithmException e)
        {
            e.printStackTrace();
        }
        catch (UnsupportedEncodingException e)
        {
            e.printStackTrace();
        }

        ret.put("url", url);
        ret.put("jsapi_ticket", jsapi_ticket);
        ret.put("nonceStr", nonce_str);
        ret.put("timestamp", timestamp);
        ret.put("signature", signature);

        return ret;
    }

    private static String byteToHex(final byte[] hash) {
        Formatter formatter = new Formatter();
        for (byte b : hash)
        {
            formatter.format("%02x", b);
        }
        String result = formatter.toString();
        formatter.close();
        return result;
    }

    private static String create_nonce_str() {
        return UUID.randomUUID().toString();
    }

    private static String create_timestamp() {
        return Long.toString(System.currentTimeMillis() / 1000);
    }






}
